#include <iostream>
#include <vector>
#include <map>
#include <utility>
#include <cmath>

int n, w, h, s;

std::map<char, std::vector<std::vector<char>>> dict;

void fillDict()
{
	char buffer[31];
	
	for (int charIndex = 0; charIndex < n; charIndex++) {
		char ch;
		std::vector<std::vector<char>> pixels (h, std::vector<char>(w, '.'));
		
		std::cin >> ch;
		for (int line = 0; line < h; line++) {
			std::cin >> buffer;
			for (int column = 0; column < w; column++) {
				pixels[line][column] = buffer[column];
			}
		}
		
		dict[ch] = std::move(pixels); 
	}
}

int countSwitchesPerLine(const std::vector<char>& line)
{
	// Before showing a character, there's either an empty column or screen is turned off.
	char currentState = '.';
	int switches = 0;
	
	for (char ch : line) {
		if (currentState != ch) {
			switches++;
			currentState = ch;
		}
	}
	// After a character, there's either an empty column or screen turns off.
	if (currentState != '.') {
		switches++;
	}
	
	return switches;
}

// Selects a character, which has a line, where the most switching occurs.
// Also returns the amount of switching, via pointer parameter.
char findMostSwitchingChar(int* switchesPerChar)
{
	int maxSwitchesPerLine = 0;
	char mostSwitchingChar;
	
	for (const auto& [ch, pixels] : dict) {
		for (const auto& line : pixels) {
			int switchesPerLine = countSwitchesPerLine(line);
			if (switchesPerLine > maxSwitchesPerLine) {
				maxSwitchesPerLine = switchesPerLine;
				mostSwitchingChar = ch;
			}
		}
	}
	*switchesPerChar = maxSwitchesPerLine;
	return mostSwitchingChar;
}

int main()
{
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(0);
	std::cout.tie(0);
	
	std::cin >> n >> w >> h >> s;
	fillDict();
	
	int nSwitchesPerChar;
	char mostSwitchingChar = findMostSwitchingChar(&nSwitchesPerChar);
	int nCharsNeeded = std::ceil((double)s / nSwitchesPerChar);
	
	// Just spam the same most effective character
	for (int i = 0; i < nCharsNeeded; i++) {
		std::cout << mostSwitchingChar;
	}
	
	return 0;
}
