#include <iostream>
#include <vector>
#include <utility>
#include <cmath>

int d;
double p;
std::vector<int> c;

double calculateA(int first, int last)
{
	int n = last - first;
	int sumOfCostChanges = 0;
	for (int i = first + 1; i <= last; i++) {
		sumOfCostChanges += c[i] - c[i-1];
	}
	return (double)sumOfCostChanges / n;
}

double calculateD(int first, int last, double a)
{
	int n = last - first;
	double sum = 0.0;
	for (int i = first + 1; i <= last; i++) {
		sum += std::pow(c[i] - c[i-1] - a,   2);
	}
	return std::sqrt(sum / n);
}

// returns  1 if positive,  -1 if negative,  or 0 if none.
int calculateTrend(double a, double d)
{
	if ((d == 0 && a > 0) || (a/d >= p)) {
		return 1;
	}
	else if ((d == 0 && a < 0) || (a/d <= -p)) {
		return -1;
	}
	else {
		return 0;
	}
}

// Need to return 2 values, so it's done via parameters
void count(int& nPositiveSegments, int& nNegativeSegments)
{
	nPositiveSegments = nNegativeSegments = 0;
	
	// Go through all possible segments, iterating over segment lengths first
	for (int length = 3; length <= d; length++) {
		int n = length - 1;
		for (int i = 0; i <= d - length; i++) {
			int j = i + length - 1;
			double a;
			if (i == 0) { // The first segment is calculated without optimizations
				a = calculateA(i, j);
			}
			else { // But the rest of the segments with that length have optimized calculation of A
				a -= ((double)(c[i] - c[i-1]) / n);
				a += ((double)(c[j] - c[j-1]) / n);
			}
			
			double d = calculateD(i, j, a);
			int trend = calculateTrend(a, d);
			
			if (trend > 0) {
				nPositiveSegments++;
			}
			else if (trend < 0) {
				nNegativeSegments++;
			}
		}
	}
}

int main()
{
	std::ios_base::sync_with_stdio(false);
	std::cin.tie(0);
	std::cout.tie(0);
	
	std::cin >> d >> p;
	c = std::move(std::vector<int>(d, 0));
	for (int i = 0; i < d; i++) {
		std::cin >> c[i];
	}
	
	int nPositiveSegments, nNegativeSegments;
	count(nPositiveSegments, nNegativeSegments);
	
	std::cout << nPositiveSegments << ' ' << nNegativeSegments;
	return 0;
}
