#include <iostream>
#include <vector>
#include <map>
#include <set>
#include <stack>

using namespace std;


void dfs(int v, vector<bool> &was, map<int, vector<int>> &gr) {
    if (was[v]) {
        return;
    }
    was[v] = true;
    for (int i : gr[v]) {
        dfs(i, was, gr);
    }
}

bool checkForEuler(map<int, vector<int>> &gr, vector<int> const & vs, map<int, int> &cntIn) {
    bool foundFirst = false, foundLast = false;
    for (int i : vs) {
        if (gr[i].size() != cntIn[i]) {
            if (!foundFirst && gr[i].size() - cntIn[i] == 1) {
                foundFirst = 1;
            } else if (!foundLast && gr[i].size() - cntIn[i] == -1) {
                foundLast = 1;
            } else {
                return false;
            }
        }
    }

    vector<bool> was(vs.size(), false);
    for (int v : vs) {
        if (!gr[v].empty()) {
            dfs(v, was, gr);
            break;
        }
    }

    for (int v : vs) {
        if (!gr[v].empty() && !was[v]) {
            return false;
        }
    }

    return true;
}

void findEulerWay(int c, map<int, vector<int>> &gr, vector<int> const & vs, vector<int> &ans,
                  map<int, vector<int>> &num, map<int, vector<int>> &failed) {
    stack<int> s;
    s.push(c);
    map<int, int> lim;
    int time = 1;
    while (!s.empty()) {
        int w = s.top();
        if (lim[w] < gr[w].size()) {
            for (int i : failed[gr[w][lim[w]]]) {
                if (ans[i] == -1) {
                    ans[i] = time++;
                }
            }
            ans[num[w][lim[w]]] = time++;
            for (int i : failed[w]) {
                if (ans[i] == -1) {
                    ans[i] = time++;
                }
            }
            s.push(gr[w][lim[w]++]);
        } else {
            s.pop();
        }
    }
}

int main()
{
    int n, c;
    cin >> n >> c;
    c--;
    map<int, vector<int>> gr;
    map<int, vector<int>> edNum;
    map<int, vector<int>> failed;
    set<int> exists;
    map<int, int> cntIn;

    for (int i = 0; i < n; i++) {
        int a, b, w;
        cin >> a >> b >> w;
        a--; b--;
        exists.insert(a);
        exists.insert(b);
        if (w) {
            gr[a].push_back(b);
            edNum[a].push_back(i);
            cntIn[b]++;
        }
        else {
            failed[a].push_back(i);
        }
    }
    vector<int> vertexes;
    while (!exists.empty()) {
        vertexes.push_back(*exists.begin());
        exists.erase(exists.begin());
    }

    vector<int> time(n, -1);

    if (checkForEuler(gr, vertexes, cntIn)) {
        findEulerWay(c, gr, vertexes, time, edNum, failed);
        for (int i : time) {
            if (i == -1) {
                cout << "No";
                return 0;
            }
        }
        for (int i : time) {
            cout << i << " ";
        }
    } else {
        cout << "No" << endl;
    }
    return 0;
}