#include <iostream>
#include <vector>
#include <algorithm>
#include <cstdint>
#include <stdint-gcc.h>
#include <cmath>
#include <unordered_map>


using namespace std;

struct edge {
    int to;
    int id;
    bool deleted;

    edge(int to, int id, bool deleted) : to(to), id(id), deleted(deleted) {}
};


vector<vector<edge>> gr;
vector<int> path;

void findEulerPath(int i) {
    for (auto &v : gr[i]) {
        if (v.deleted)
            continue;
        int b = v.to;
        v.deleted = true;
        findEulerPath(b);
    }
    path.push_back(i);
}

void dfs(int i, vector<bool> &visited) {
    for (auto v : gr[i]) {
        if (!visited[v.id]) {
            visited[v.id] = true;
            dfs(v.to, visited);
        }
    }
}

bool oneComp(int c, int w1_size) {
    vector<bool> visited(gr.size());
    dfs(c, visited);
    int count = 0;
    for (bool b : visited) {
        if (b)
            count++;
    }
    return count == w1_size;
}

int main() {
    int n, c;
    cin >> n >> c;
    gr.resize(2 * n + 1);
    vector<int> degIn(2 * n + 1);
    vector<int> degOut(2 * n + 1);
    vector<pair<int, int>> w0;
    unordered_map<int, int> compr;
    int nextNum = 0;
    int w1_size = 0;
    for (int i = 0; i < n; i++) {
        int a, b, w;
        cin >> a >> b >> w;
        if (compr.find(a) == compr.end()) {
            compr.emplace(a, nextNum);
            nextNum++;
        }
        if (compr.find(b) == compr.end()) {
            compr.emplace(b, nextNum);
            nextNum++;
        }
        a = compr.find(a)->second;
        b = compr.find(b)->second;
        if (w == 1) {
            gr[a].emplace_back(b, i, false);
            degOut[a]++;
            degIn[b]++;
            w1_size++;
        } else {
            w0.emplace_back(a, i);
        }
    }
    if (compr.find(c) == compr.end()) {
        compr.emplace(c, nextNum);
        nextNum++;
    }
    c = compr.find(c)->second;
    bool isEuler = true;
    if (degOut[c] != degIn[c] + 1 && degOut[c] != degIn[c])
        isEuler = false;
    int end = -1;
    for (int i = 0; i < nextNum && isEuler; i++) {
        if (i == c) {
            continue;
        }
        if (degIn[i] == degOut[i]) {
            continue;
        }
        if (end == -1 && degOut[i] == degIn[i] - 1 && degOut[c] == degIn[c] + 1) {
            end = i;
        } else {
            isEuler = false;
        }
    }
    if (!oneComp(c, w1_size)) {
        isEuler = false;
    }
    if (end == -1 && degOut[c] == degIn[c] + 1)
        isEuler = false;
    if (!isEuler) {
        cout << "NO\n";
    } else {
        findEulerPath(c);
        int count = w0.size();
        vector<int> ans;
        vector<bool> visited(n);
        for (auto &i : gr) {
            for (auto &e : i)
                e.deleted = false;
        }
        for (int i = path.size() - 1; i >= 0; i--) {
            int now = path[i];
            if (count != 0) {
                for (auto v : w0) {
                    if (v.first != now && !visited[v.second]) {
                        ans.push_back(v.second + 1);
                        visited[v.second] = true;
                        count--;
                    }
                }
            }
            if (i == 0)
                continue;
            for (auto &e : gr[now]) {
                if (!e.deleted && e.to == path[i - 1]) {
                    ans.push_back(e.id + 1);
                    e.deleted = true;
                    break;
                }
            }
        }
        if (count != 0) {
            cout << "NO\n";
        } else {
            cout << "YES\n";
            for (int a : ans) {
                cout << a << " ";
            }
        }
    }
}
