﻿#pragma GCC optimize("O3")

#include <iostream>
#include <algorithm>
#include <vector>
#include <string>
#include <cstring>
#include <numeric>
#include <iomanip>
#include <tuple>
#include <random>
#include <chrono>
#include <set>
#include <map>
#include <unordered_set>
#include <unordered_map>
#include <queue>
#include <bitset>
#include <cmath>
#include <cassert>

using namespace std;
using ll = long long;
using ld = long double;

const int inf = 1e9 + 228;

struct Op {
	int u, v, w;
};

int n, c;
vector<Op> ops;

void read(istream& cin) {
	cin >> n >> c;
	ops.resize(n);
	for (auto& op : ops) {
		cin >> op.u >> op.v >> op.w;
	}
}

void no() {
	cout << "No" << '\n';
	exit(0);
}

struct DSU {
	vector<int> p, s;
	DSU(int n) : p(n), s(n, 1) { iota(p.begin(), p.end(), 0); }
	int get(int v) {
		return (v == p[v] ? v : p[v] = get(p[v]));
	}
	bool unite(int u, int v) {
		if ((u = get(u)) != (v = get(v))) {
			if (s[u] < s[v]) {
				swap(u, v);
			}
			s[u] += s[v];
			p[v] = u;
			return true;
		}
		else {
			return false;
		}
	}
};

void solve(ostream& cout) {
	int lo = c, hi = c;
	vector<int> vs = { c };
	for (auto op : ops) {
		if (op.w == 0) continue;
		vs.push_back(op.u);
		vs.push_back(op.v);
		lo = min({ lo, op.u, op.v });
		hi = max({ hi, op.u, op.v });
	}
	sort(vs.begin(), vs.end());
	vs.erase(unique(vs.begin(), vs.end()), vs.end());
	vector<set<int>> g(vs.size());
	vector<int> inDeg(vs.size());
	DSU dsu(vs.size());
	int cnt = 0;
	for (auto op : ops) {
		if (op.w == 0) continue;
		int v = lower_bound(vs.begin(), vs.end(), op.u) - vs.begin();
		int to = lower_bound(vs.begin(), vs.end(), op.v) - vs.begin();
		g[v].insert(to);
		inDeg[to]++;
		cnt += dsu.unite(v, to);
	}
	if (cnt + 1 != vs.size()) {
		no();
	}
	int cbad = 0;
	bool badIn = false;
	bool badOut = false;
	int _st = -1;
	for (int u = 0; u < g.size(); u++) {
		if (g[u].size() != inDeg[u]) {
			cbad++;
			int in = inDeg[u];
			int out = g[u].size();
			int dif = in - out;
			if (dif == 1) {
				badIn = true;
			}
			else if (dif == -1) {
				badOut = true;
				_st = u;
			}
			else {
				no();
			}
		}
	}
	if (cbad != 0 && cbad != 2) {
		no();
	}
	if (cbad == 2 && (!badIn || !badOut || _st == -1 || vs[_st] != c)) {
		no();
	}
	_st = lower_bound(vs.begin(), vs.end(), c) - vs.begin();
	vector<int> st = { _st };
	vector<int> path;
	while (!st.empty()) {
		int v = st.back();
		if (g[v].empty()) {
			path.push_back(v);
			st.pop_back();
		}
		else {
			int to = *g[v].begin();
			g[v].erase(g[v].begin());
			st.push_back(to);
		}
	}
	reverse(path.begin(), path.end());
	if (vs[path[0]] != c) {
		int i = 0;
		while (st[path[i]] != c) {
			i++;
		}
		rotate(path.begin(), path.begin() + i, path.end());
	}
	map<pair<int, int>, int> renum;
	vector<int> nonLow, nonHi;
	for (int i = 0; i < ops.size(); i++) {
		auto op = ops[i];
		if (op.w == 1) {
			int u = lower_bound(vs.begin(), vs.end(), op.u) - vs.begin();
			int to = lower_bound(vs.begin(), vs.end(), op.v) - vs.begin();
			renum[{ u, to }] = i;
		}
		else {
			if (op.u != lo) {
				nonLow.push_back(i);
			}
			else if(op.u != hi) {
				nonHi.push_back(i);
			}
			else {
				no();
			}
		}
	}
	cout << "Yes\n";
	int lst = -1;
	for (int v : path) {
		if (lst != -1) {
			cout << renum[{ lst, v }] + 1 << ' ';
		}
		if (vs[v] == lo) {
			for (int x : nonLow) {
				cout << x + 1 << ' ';
			}
		}
		else if (vs[v] == hi) {
			for (int x : nonHi) {
				cout << x + 1 << ' ';
			}
		}
		lst = v;
	}
	cout << '\n';
}

int32_t main() {
	ios::sync_with_stdio(false);
	cin.tie(nullptr);
	int t = 1;
	// cin >> t;
	while (t--) {
		read(cin);
		solve(cout);
	}
}
