#ifdef LOCAL
#define _GLIBCXX_DEBUG
#endif

#include <bits/stdc++.h>
//#define assert(...) void(0)

using namespace std;

using ll = long long;
using ull = unsigned long long;

using ld = long double;

mt19937_64 mt(736736);

const int digits = 18;
const ll base = ll(1e18);


struct bigint
{
	vector<ll> digs;

	explicit bigint()
	{
	}

	explicit bigint(const int x)
	{
		digs.push_back(x);
	}

	void make_inf()
	{
		digs = {-1};
	}

	bool is_inf() const
	{
		return !digs.empty() && digs[0] == -1;
	}

	void relax ()
	{
		for (int i = 0; i < int(digs.size()); i++)
			if (digs[i] >= base)
			{
				assert(digs[i] < 2 * base);
				if (i + 1 == int(digs.size()))
					digs.push_back(0);
				digs[i + 1]++;
				digs[i] -= base;
			}
	}

	string to_string () const
	{
		if (is_inf())
			return "NO";
		if (digs.empty())
			return "0";

		string ans;
		for (int i = int(digs.size()) - 1; i >= 0; i--)
		{
			stringstream ss;
			ss << digs[i];
			string cur;
			ss >> cur;

			if (!ans.empty())
				cur = string(digits - int(cur.size()), '0') + cur;
			ans += cur;
		}

		return ans;
	}
};

bool operator<(const bigint &l, const bigint &r)
{
	if (l.is_inf())
		return false;
	if (r.is_inf())
		return true;

	if (l.digs.size() != r.digs.size())
		return l.digs.size() < r.digs.size();

	for (int i = int(l.digs.size()) - 1; i >= 0; i--)
		if (l.digs[i] != r.digs[i])
			return l.digs[i] < r.digs[i];

	return false;
}

bool operator == (const bigint &l, const bigint &r)
{
	if (l.digs.size() != r.digs.size())
		return false;
	return l.digs == r.digs;
}

bigint operator+(const bigint &l, const bigint &r)
{
	if (l.is_inf() || r.is_inf())
		return bigint(-1);

	bigint ans;
	ans.digs.resize(max(l.digs.size(), r.digs.size()));

	for (int it = 0; it < 2; it++)
	{
		const auto &cur = (it == 0 ? l.digs : r.digs);
		for (int i = 0; i < int(cur.size()); i++)
			ans.digs[i] += cur[i];
	}

	ans.relax();
	return ans;
}

bool remin(bigint &cur, const bigint &what)
{
	if (what.is_inf())
		return false;
	if (cur.is_inf() || cur < what)
	{
		cur = what;
		return true;
	}

	return false;
}

const int let = 26;

template<class T>
using min_heap = priority_queue<T, vector<T>, greater<T>>;

tuple<int, int, int> decode (const int v)
{
	return tuple<int, int, int>{v / let / let, v / let % let, v % let};
}

int encode (const int s, const int t, const int n)
{
	return s * let * let + t * let + n;
}

void solve(istream &cin = std::cin, ostream &cout = std::cout)
{
	const bigint inf = bigint(-1);

	vector<vector<int>> who_simple(let);
	vector<vector<pair<int, int>>> left_part(let), right_part(let), split(let);

	int rules;
	cin >> rules;

	for (int i = 0; i < rules; i++)
	{
		string left, to, right;
		cin >> left >> to >> right;
		assert(to == "->");
		assert(left.size() == 1U);
		assert('A' <= left.front() && left.front() <= 'Z');

		const int who_split = int(left.front() - 'A');

		if (right.size() == 1U)
		{
			assert('a' <= right.front() && right.front() <= 'z');
			who_simple[right.front() - 'a'].push_back(who_split);
		}
		else if (right.size() == 2U)
		{
			const int who_left = int(right.front() - 'A');
			const int who_right = int(right.back() - 'A');
			assert(0 <= min(who_left, who_right) && max(who_left, who_right) < let);
			split[who_split].emplace_back(who_left, who_right);
			left_part[who_left].emplace_back(who_right, who_split);
			right_part[who_right].emplace_back(who_left, who_split);
		}
		else
			assert(false);
	}

	int n, edges, s, t;
	cin >> n >> edges >> s >> t;
	--s, --t;

	vector<vector<int>> symb(n, vector<int>(n, -1));
	for (int i = 0; i < edges; i++)
	{
		int u, v;
		char x;
		cin >> u >> v >> x;
		--u, --v;
		assert('a' <= x && x <= 'z');
		symb[u][v] = int(x - 'a');
	}

	vector<bigint> dist(let * let * let, inf);

	for (int i = 0; i < n; i++)
		for (int j = 0; j < n; j++)
			if (symb[i][j] != -1)
				for (const int k : who_simple[symb[i][j]])
					dist[encode(i, j, k)] = bigint(1);

	auto cmp = [&] (const int x, const int y)
	{
		return (dist[x] < dist[y]) || (dist[x] == dist[y] && x < y);
	};

	set<int, decltype(cmp)> S(cmp);

	for (int i = 0; i < int(dist.size()); i++)
		if (!dist[i].is_inf())
			S.insert(i);

	while (!S.empty())
	{
		const int ver = *S.begin();
		S.erase(S.begin());

		int u, v, a;
		tie(u, v, a) = decode(ver);

		for (int w = 0; w < n; w++)
		{
			for (int it = 0; it < 2; it++)
			{
				const auto &cur_part = (it == 0 ? left_part[a] : right_part[a]);

				for (const auto &pr : cur_part)
				{
					const bigint &new_val = dist[encode(it == 0 ? v : w, it == 0 ? w : u, pr.first)] + dist[ver];
					if (new_val.is_inf())
						continue;

					const int to = encode(it == 0 ? u : w, it == 0 ? w : v, pr.second);
					bigint &old_val = dist[to];

					if (new_val < old_val)
					{
						S.erase(to);
						old_val = new_val;
						S.insert(to);
					}
				}
			}
		}
	}

//  assert(!split['S' - 'A'].empty());
	const string ans = dist[encode(s, t, 'S' - 'A')].to_string();
    assert(ans != "0");
	cout << ans << endl;
}

int main()
{
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);

#ifdef LOCAL
	auto st = clock();

	ifstream fin("../input.txt");

	do
	{
		solve(fin);

		cout << "===" << endl;

		string str;
		while (getline(fin, str) && str != string(max(1, (int) str.size()), '='));
	} while (fin);

	cout << setprecision(6) << "clock: " << double(clock() - st) / CLOCKS_PER_SEC << endl;
#else
	solve();
#endif

	return 0;
}