#include <bits/stdc++.h>
#define int long long
using namespace std;
typedef long double ld;

const int inf = 1e18;
const ld eps = 1e-9;
const ld pi = atan2(0, -1);

struct Operation {
    int a, b, w, id;

    Operation() {}
    Operation(int a, int b, int w, int id): a(a), b(b), w(w), id(id) {

    }
};

struct Solver {
    vector<vector<Operation>> g;
    vector<int> ptr, res;
    vector<Operation> a;
    int n, s, gg, path_size;
    vector<int> bad_edges;
    int bad_ptr;

    Solver(vector<Operation> a, int s): a(a), s(s) {
        gg = 0, path_size = a.size();
        bad_ptr = 0;

        n = 0;
        for(auto i : a)
            n = max(max(i.a, i.b), n);
        n++;

        g.assign(n, {});
        ptr.assign(n, 0);

        for(auto e : a) {
            if(e.w) {
                g[e.a].push_back(e);
            }

            else {
                bad_edges.push_back(e.id);
            }
        }
    }

    void find_path(int v, int id) {
        for(int &i = ptr[v]; i < g[v].size(); ++i) {
            auto e = g[v][i];
            i++;
            find_path(e.b, e.id);
        }

        for(int& i = bad_ptr; i < bad_edges.size(); ++i) {
            if(a[bad_edges[i]-1].a == v) break;
            res.push_back(a[bad_edges[i]-1].id);
        }

        if(id > 0) res.push_back(id);
    }

    vector<int> solve() {
        if(gg) return {};
        sort(bad_edges.begin(), bad_edges.end(), [this](int u, int v) {
            return a[u-1].a > a[v-1].a;
        });
        find_path(s, -1);
        reverse(res.begin(), res.end());

//        for(int i : res)
//            cout << i << " ";
//        cout << "\n";

        int v = s;
        for(int j : res) {
            auto i = a[j-1];
            if(i.w) {
                if(v != i.a) return {};
                v = i.b;
            }

            else {
                if(v == i.a) return {};
            }
        }

        if(res.size() == path_size)
            return res;
        else
            return {};
    }
};

pair<vector<Operation>, int> compress(vector<Operation> a, int s) {
    vector<int> num(2*a.size(), 0);
    int j = 0;
    for(auto i : a) {
        num[j++] = i.a;
        num[j++] = i.b;
    }

    sort(num.begin(), num.end());
    num.erase(unique(num.begin(), num.end()), num.end());

    for(int i = 0; i < a.size(); ++i) {
        a[i].a = lower_bound(num.begin(), num.end(), a[i].a)-num.begin();
        a[i].b = lower_bound(num.begin(), num.end(), a[i].b)-num.begin();
    }

    s = lower_bound(num.begin(), num.end(), s)-num.begin();

    return {a, s};
}

const int debug = 0, fast_io = 1;
int32_t main() {
//    freopen("inp.txt", "r", stdin);
    cout << fixed << setprecision(16);
    if(fast_io) {
        ios_base::sync_with_stdio();
        cin.tie(0); cout.tie(0);
    }

    int tests = (debug ? 1000 : 1);
//    cin >> tests;

    for(int test = 0; test < tests; ++test) {
        if(!debug) {
            /// Main part ///

            int n, s;
            cin >> n >> s;

            vector<Operation> a(n);
            for(int i = 0; i < n; ++i) {
                int ai, bi, wi;
                cin >> ai >> bi >> wi;
                a[i] = {ai, bi, wi, i+1};
            }

            auto compressed = compress(a, s);
            a = compressed.first;
            s = compressed.second;

            Solver solver(a, s);
            vector<int> res = solver.solve();

            if(res.size() > 0) {
                cout << "Yes\n";
                for(int i : res)
                    cout << i << " ";
            }

            else {
                cout << "No\n";
            }

            /// Main part ///
        } else {}
    }

    return 0;
}

/**

*/
