#include <iostream>
#include <string>
#include <functional>
#include <map>
#include <array>
#include <set>
#include <vector>
#include <algorithm>
#include <cstdlib>

using namespace std;

using ll = long long;

array<bool, 256> tt;
array<bool, 256> initial;

struct AST {
    string s;
    AST *l, *r;
};

AST *term();
AST *factor();
AST *element();

bool eval(AST *ast, array<bool, 8> vals) {
    if (ast->s.size() == 1) {
        return vals[ast->s[0] - 'a'];
    }

    if (ast->s == "and") {
        return eval(ast->l, vals) && eval(ast->r, vals);
    }

    if (ast->s == "or") {
        return eval(ast->l, vals) || eval(ast->r, vals);
    }

    if (ast->s == "not") {
        return !eval(ast->l, vals);
    }
    return false;
}

void build() {
    auto ast = term();

    for (int i = 0; i < 256; ++i) {
        array<bool, 8> vals;
        for (int j = 0; j < 8; ++j) {
            if ((i >> j) & 1) {
                vals[j] = 1;
            } else {
                vals[j] = 0;
            }
        }
        tt[i] = eval(ast, vals);
    }
}

void check() {
    initial = tt;

    for (int i = 0; i < 256; ++i) {
        for (int j = 1; j < 256; ++j) {
            if (tt[i] && !tt[i | j]) {
                cout << "IMPOSSIBLE\n";
                exit(0);
            }

            if (tt[i] && (i | j) != i) {
                initial[i | j] = false;
            }
        }
    }
}

int tokptr = 0;
vector<string> tokens;

string read() {
    return tokens[tokptr++];
}

string peek() {
    return tokens[tokptr];
}

AST *term() {
    auto f = factor();
    if (peek() == "or") {
        read();
        return new AST{ "or", f, term() };
    }
    return f;
}

AST *factor() {
    auto e = element();
    if (peek() == "and") {
        read();
        return new AST{ "and", e, factor() };
    }
    return e;
}

AST *element() {
    if (peek() == "(") {
        read();
        auto t = term();
        read();
        return t;
    }

    if (peek() == "not") {
        read();
        return new AST{ "not", element(), nullptr };
    }

    return new AST{ read(), nullptr, nullptr };
}

int main() {
    cin.tie(0);
    ios::sync_with_stdio(false);

    string s;
    getline(cin, s);
    string cur;
    for (char c : s) {
        if (c == '(') {
            if (cur != "") tokens.push_back(cur);
            tokens.push_back("(");
            cur = "";
            continue;
        }

        if (c == ')') {
            if (cur != "") tokens.push_back(cur);
            tokens.push_back(")");
            cur = "";
            continue;
        }

        if (c == ' ') {
            if (cur != "") tokens.push_back(cur);
            cur = "";
            continue;
        }

        cur += c;
    }
    if (cur != "") tokens.push_back(cur);
    tokens.push_back("EOF");

    build();
    check();
    vector<ll> zz;
    for (int i = 0; i < 256; ++i) if (initial[i]) zz.push_back(i);
    if (zz.empty()) {
        cout << "++\n";
        return 0;
    }

    for (int j = 0; j < 8; ++j) {
        for (int i : zz) {
            cout << "+";
            if ((i >> j) & 1) {
                cout << (char)('a' + j);
            } else {
                cout << ' ';
            }
        }
        cout << "+\n";
    }

    return 0;
}
