#include <iostream>
#include <string>
#include <functional>
#include <map>
#include <array>
#include <set>
#include <vector>
#include <algorithm>
#include <cstdlib>

using namespace std;

using ll = long long;

array<bool, 256> tt;
array<bool, 256> initial;

struct AST {
    string s;
    AST *l, *r;
};

AST *term();
AST *factor();
AST *element();

bool eval(AST *ast, array<bool, 8> vals) {
    if (ast->s.size() == 1) {
        return vals[ast->s[0] - 'a'];
    }

    if (ast->s == "and") {
        return eval(ast->l, vals) && eval(ast->r, vals);
    }

    if (ast->s == "or") {
        return eval(ast->l, vals) || eval(ast->r, vals);
    }

    if (ast->s == "not") {
        return !eval(ast->l, vals);
    }
    return false;
}

void build() {
    auto ast = term();

    for (int i = 0; i < 256; ++i) {
        array<bool, 8> vals;
        for (int j = 0; j < 8; ++j) {
            if ((i >> j) & 1) {
                vals[j] = 1;
            } else {
                vals[j] = 0;
            }
        }
        tt[i] = eval(ast, vals);
    }
}

void check() {
    initial = tt;

    for (int i = 0; i < 256; ++i) {
        for (int j = 1; j < 256; ++j) {
            if (tt[i] && !tt[i | j]) {
                cout << "IMPOSSIBLE\n";
                exit(0);
            }

            if (tt[i] && (i | j) != i) {
                initial[i | j] = false;
            }
        }
    }
}

int tokptr = 0;
vector<string> tokens;

string read() {
    return tokens[tokptr++];
}

string peek() {
    return tokens[tokptr];
}

AST *term() {
    auto f = factor();
    if (peek() == "or") {
        read();
        return new AST{ "or", f, term() };
    }
    return f;
}

AST *factor() {
    auto e = element();
    if (peek() == "and") {
        read();
        return new AST{ "and", e, factor() };
    }
    return e;
}

AST *element() {
    if (peek() == "(") {
        read();
        auto t = term();
        read();
        return t;
    }

    if (peek() == "not") {
        read();
        return new AST{ "not", element(), nullptr };
    }

    return new AST{ read(), nullptr, nullptr };
}

int main() {
    cin.tie(0);
    ios::sync_with_stdio(false);

    string s;
    getline(cin, s);
    string cur;
    for (char c : s) {
        if (c == '(') {
            if (cur != "") tokens.push_back(cur);
            tokens.push_back("(");
            cur = "";
            continue;
        }

        if (c == ')') {
            if (cur != "") tokens.push_back(cur);
            tokens.push_back(")");
            cur = "";
            continue;
        }

        if (c == ' ') {
            if (cur != "") tokens.push_back(cur);
            cur = "";
            continue;
        }

        cur += c;
    }
    if (cur != "") tokens.push_back(cur);
    tokens.push_back("EOF");

    build();
    check();
    vector<vector<ll>> zz;
    for (int i = 0; i < 256; ++i) {
        if (!initial[i]) continue;
        vector<ll> temp;
        for (int j = 0; j < 8; ++j) {
            if ((i >> j) & 1) {
                temp.push_back(j);
            }
        }
        if (temp.empty()) {
            cout << "+ +\n";
            return 0;
        }
        zz.push_back(temp);
    }

    if (zz.empty()) {
        cout << "++\n";
        return 0;
    }

    vector<vector<char>> field(50, vector<char>(50, ' '));

    if (zz.size() < 23) {
        ll mx = 1;
     
        for (auto z : zz) {
            if (z.size() > mx) mx = z.size();
        }
     
        for (int j = 0; j < mx; ++j) {
            int x = 0;
            int last = 0;
            for (auto i : zz) {
                int cur = (last > i.size()) ? last : i.size();
     
                if (j < cur) {
                    field[j][x++] = '+';
                } else {
                    field[j][x++] = ' ';
                }
                if (j < i.size()) {
                    field[j][x++] = (char)('a' + i[j]);
                } else {
                    field[j][x++] = ' ';
                }
                last = i.size();
            }
            if (j < last) {
                field[j][x++] = '+';
            } else {
                field[j][x++] = ' ';
            }
        }
     
        for (int y = 0; y < 50; ++y) {
            bool f = false;
            int mx = 0;
            for (int x = 0; x < 50; ++x) {
                bool l = (x > 0) ? field[y][x-1] == ' ' : true;
                bool r = (x < 49) ? field[y][x+1] == ' ' : true;
                if (field[y][x] == '+' && l && r && y > 0) {
                    field[y][x] = '|';
                }
                if (field[y][x] != ' ') {
                    f = true;
                    mx = x;
                }
            }
            if (!f) break;
            for (int x = 0; x <= mx; ++x) {
                cout << field[y][x];
            }
            cout << '\n';
        }

        return 0;
    }

    ll mx = 1;
    ll mj = 0;

    for (auto z : zz) {
        if (z.size() > mx) mx = z.size();
    }

    auto zz2 = zz;
    ll T = 20;

    for (ll i = 0; i < zz2.size(); i += T) {
        zz.clear();
        for (ll j = i; j < i + T; ++j) {
            if (j < zz2.size()) zz.push_back(zz2[j]);
        }

        bool rem = (i + T) < zz2.size();

        int xxx;

        for (int j = 0; j < mx; ++j) {
            int jj = mj + j;
            int x = 0;
            int last = 0;
            for (auto i : zz) {
                int cur = (last > i.size()) ? last : i.size();

                if (j < cur) {
                    field[jj][x++] = '+';
                } else {
                    field[jj][x++] = ' ';
                }
                if (j < i.size()) {
                    field[jj][x++] = (char)('a' + i[j]);
                } else {
                    field[jj][x++] = ' ';
                }
                last = i.size();
            }
            if (j < last) {
                field[jj][x++] = '+';
            } else {
                field[jj][x++] = ' ';
            }

            if (j == 0 || rem) {
                field[jj][x++] = '+';
            } else if (rem) {
                x++;
                field[jj][x++] = '+';
            }
            if (rem) field[jj][49] = '+';
            xxx = x;

            if (!rem && j == 0) {
                while (x < 50) field[jj][x++] = '+';
            }
        }

        if (rem) {
            field[mj+mx][xxx - 1] = '+';
            for (int i = 0; i < xxx; ++i) field[mx+mj+1][i] = '+';
            field[mx+mj+2][0] = '+';
            field[mx+mj+3][0] = '+';
            field[mx+mj][49] = '+';
            field[mx+mj+1][49] = '+';
            field[mx+mj+2][49] = '+';
            field[mx+mj+1][49] = '+';
            mj += mx + 3;
        }
    }

    for (int y = 0; y < 50; ++y) {
        bool f = false;
        int mx = 0;
        for (int x = 0; x < 50; ++x) {
            bool l = (x > 0) ? field[y][x-1] == ' ' : true;
            bool r = (x < 49) ? field[y][x+1] == ' ' : true;
            bool u = (y > 0) ? field[y-1][x] == ' ' : true;
            bool d = (y < 49) ? field[y+1][x] == ' ' : true;
            if (field[y][x] == '+' && l && r) {
                field[y][x] = '|';
            }
            if (field[y][x] == '+' && u && d) {
                field[y][x] = '-';
            }
            if (field[y][x] != ' ') {
                f = true;
                mx = x;
            }
        }
        field[0][0] = '+';
        field[0][49] = '+';
        if (!f) break;
        for (int x = 0; x <= mx; ++x) {
            cout << field[y][x];
        }
        cout << '\n';
    }

    return 0;
}
